package com.warewms.ams.common;

import cn.hutool.core.comparator.CompareUtil;

import java.util.Arrays;

public class AmsConstant {

    /**
     * 出库缓存位
     */
    public static final Long LOC_SORTATION_CACHE = 99998l;

    /**
     * 中间缓存位
     * AGV作业搬运任务时候库存的虚拟库位
     */
    public static final Long LOC_MIDDLE_CACHE = 99999l;

    /**
     * agv点位缓存key
     */
    public static final String WMS_CORRESPOND_AGV_STATION_KEY = "wms_correspond_agv_station:";

    /**
     * 区域信息缓存key
     */
    public static final String LOCATION_ROW_KEY = "location_row:";

    /**
     * 是否下发NDC
     */
    public enum ACI_ACCEPT_STATUS {
        /**
         * 已下发
         */
        Y(1),
        /**
         * 未下发
         */
        N(0);

        private Integer value;

        ACI_ACCEPT_STATUS(Integer value) {
            this.value = value;
        }

        public Integer getValue() {
            return value;
        }
    }

    /**
     * 仓库类型
     */
    public enum ZONE_TYPE {

        /**
         * 库位区
         */
        INV(1),

        /**
         * 分拣区
         */
        SORT(2),

        /**
         * 入库接驳位
         */
        IN_CACHE(3),

        /**
         * 出库接驳位
         */
        OUT_CACHE(4);

        private Integer value;

        ZONE_TYPE(Integer value) {
            this.value = value;
        }

        public Integer getValue() {
            return value;
        }
    }

    /**
     * 是否删除任务
     */
    public enum DELETE_STATUS {
        /**
         * 是
         */
        Y(1),
        /**
         * 否
         */
        N(0);

        private Integer value;

        DELETE_STATUS(Integer value) {
            this.value = value;
        }

        public Integer getValue() {
            return value;
        }
    }

    /**
     * NDC任务类型
     */
    public enum TASK_BUSINESS_TYPE {
        /**
         * 01叉货任务
         */
        TASK_01("01"),
        /**
         * 60暂停点
         */
        TASK_60("60"),
        /**
         * 90暂停重启
         */
        TASK_90("90"),
        /**
         * 电量
         */
        TASK_POWER("power"),
        /**
         * m
         */
        TASK_m("m"),
        /**
         * 按钮盒触发交通管制
         */
        TASK_74("74"),
        /**
         * 按钮盒解除交通管制
         */
        TASK_75("75"),
        /**
         * 光栅触发交通管制
         */
        TASK_76("76"),
        /**
         * 光栅解除交通管制
         */
        TASK_77("77"),
        /**
         * 暂停
         */
        TASK_03("03");

        private String value;

        TASK_BUSINESS_TYPE(String value) {
            this.value = value;
        }

        public String getValue() {
            return value;
        }
    }

    /**
     * 任务状态 State
     */
    public enum TASK_STS {
        /**
         * 下发AMS
         */
        TASK0(0, 0),
        /**
         * 开始
         */
        TASK1(86, 1),
        /**
         * 完成
         */
        TASK2(90, 2),
        /**
         * 开始取货
         */
        TASK3(3, 3),
        /**
         * 取货完成
         */
        TASK4(91, 4),
        /**
         * 开始卸货
         */
        TASK5(5, 5),
        /**
         * 卸货完成
         */
        TASK6(92, 6),
        /**
         * 取消
         */
        TASK7(85, 7),
        /**
         * 空叉取消
         */
        TASK8(113, 8),
        /**
         * 队列等待
         */
        TASK9(9, 9),
        /**
         * 已下发
         */
        TASK10(10, 10);

        private Integer key;

        private Integer value;

        public Integer getKey() {
            return key;
        }

        public Integer getValue() {
            return value;
        }

        TASK_STS(Integer key, Integer value) {
            this.value = value;
            this.key = key;
        }

        public static Integer getValueByKey(Integer key) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getKey(), key) == 0)
                    .map(AmsConstant.TASK_STS::getValue).findFirst().orElseGet(() -> null);
        }
    }

    /**
     * 光栅检测信号
     */
    public enum GRATING_DETECTION_SIGNAL {
        /***
         * 1号光栅
         */
        NO1("sys.guangshan.no1", "3F00010D0A"),
        /***
         * 2号光栅
         */
        NO2("sys.guangshan.no2", "3F00020D0A"),
        /***
         * 3号光栅
         */
        NO3("sys.guangshan.no3", "3F00030D0A"),
        /***
         * 4号光栅
         */
        NO4("sys.guangshan.no4", "3F00040D0A"),
        /***
         * 5号光栅
         */
        NO5("sys.guangshan.no5", "3F00510D0A"),
        /***
         * 6号光栅
         */
        NO6("sys.guangshan.no6", "3F00610D0A");

        /**
         * 光栅编号
         */
        private String key;

        /**
         * 检测信号
         */
        private String value;

        GRATING_DETECTION_SIGNAL(String key, String value) {
            this.key = key;
            this.value = value;
        }

        /**
         * 获取光栅编号
         *
         * @return
         */
        public String getKey() {
            return key;
        }

        /**
         * 获取检测信号
         *
         * @return
         */
        public String getValue() {
            return value;
        }

        /**
         * 根据光栅号获取检测信号
         *
         * @param key
         * @return
         */
        public static String getValueByKey(String key) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getKey(), key) == 0)
                    .map(AmsConstant.GRATING_DETECTION_SIGNAL::getValue).findFirst().orElseGet(() -> null);
        }
    }

    /**
     * 出入库区域信息
     */
    public enum AREA_INFO {
        /***
         * 入库区域
         */
        STORAGE_AREA("storageAreaControl", "2"),
        /***
         * 出库区域
         */
        EXIT_AREA("exitAreaControl", "1");

        /**
         * 区域缓存key
         */
        private String key;

        /**
         * 区域对接杭叉AGV LP0字段
         */
        private String value;

        AREA_INFO(String key, String value) {
            this.key = key;
            this.value = value;
        }

        /**
         * 获取区域缓存key
         *
         * @return
         */
        public String getKey() {
            return key;
        }

        /**
         * 获取区域对接杭叉AGV LP0字段
         *
         * @return
         */
        public String getValue() {
            return value;
        }

        /**
         * 根据区域缓存key获取区域对接杭叉AGV LP0字段
         *
         * @param key
         * @return
         */
        public static String getValueByKey(String key) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getKey(), key) == 0)
                    .map(AmsConstant.AREA_INFO::getValue).findFirst().orElseGet(() -> null);
        }

        /**
         * 根据区域对接杭叉AGV LP0字段获取区域缓存key
         *
         * @param value
         * @return
         */
        public static String getKeyByValue(String value) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getValue(), value) == 0)
                    .map(AmsConstant.AREA_INFO::getKey).findFirst().orElseGet(() -> null);
        }
    }

    /**
     * 出入库区域光栅解除信号
     */
    public enum GRATING_RELIEVE_CMD {
        /***
         * 入库区域光栅
         */
        STORAGE_GRATING("storageAreaGrating", "3F00500D0A"),
        /***
         * 出库区域光栅
         */
        EXIT_GRATING("exitAreaGrating", "3F00600D0A");

        /**
         * 区域缓存key
         */
        private String key;

        /**
         * 接触信号
         */
        private String value;

        GRATING_RELIEVE_CMD(String key, String value) {
            this.key = key;
            this.value = value;
        }

        /**
         * 获取区域缓存key
         *
         * @return
         */
        public String getKey() {
            return key;
        }

        /**
         * 获取区域光栅解除信号
         *
         * @return
         */
        public String getValue() {
            return value;
        }

        /**
         * 根据区域缓存key获取区域光栅解除信号
         *
         * @param key
         * @return
         */
        public static String getValueByKey(String key) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getKey(), key) == 0)
                    .map(GRATING_RELIEVE_CMD::getValue).findFirst().orElseGet(() -> null);
        }

        /**
         * 根据区域对接杭叉AGV LP0字段获取区域缓存key
         *
         * @param value
         * @return
         */
        public static String getKeyByValue(String value) {
            return Arrays.stream(values()).filter(item -> CompareUtil.compare(item.getValue(), value) == 0)
                    .map(GRATING_RELIEVE_CMD::getKey).findFirst().orElseGet(() -> null);
        }
    }
}
