package com.warewms.ams.ndc;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.common.Aci;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.ams.ndc.domain.AmsHexdefineDetail;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.entity.Order;
import com.warewms.ams.ndc.service.IAmsHexdefineDetailService;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.ams.ndc.service.SocketBufferService;
import com.warewms.common.exception.base.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: andy.qu
 * Date: 2022/8/5
 */
@Slf4j
@Service
public class AciService {

    private static final int TIMEOUT = 3000;

    private static final int BUFF_SIZE = 2048;

    private static final String IP = "127.0.0.1";

    private Selector selector;

    private SocketChannel socketChannel;

    private static final int PORT = 30002;

    @Autowired
    private SocketBufferService socketBufferService;
    @Autowired
    private IAmsTaskService amsTaskService;
    @Autowired
    private IAmsHexdefineDetailService amsHexdefineDetailService;

    public void init() throws IOException {
        selector = Selector.open();
        try {
            socketChannel = socketChannel.open(new InetSocketAddress(IP, PORT));
            socketChannel.configureBlocking(false);
            socketChannel.register(selector, SelectionKey.OP_READ);
        } catch (Exception exception) {
            log.warn("AciService socketChannel open is on error");
            try {
                Thread.currentThread().sleep(3000);
            }catch (InterruptedException e) {
                throw new BaseException(e.getMessage());
            }
            init();
        }
    }

    @SuppressWarnings("InfiniteLoopStatement")
    public void start() throws IOException {
        init();
        readInfo();
        while (true) {
            try {
                ByteBuffer buffer = ByteBuffer.allocate(BUFF_SIZE);
                buffer.clear();
                //删除指令
                AmsTask taskBean = new AmsTask();
                taskBean.setIsDelete(1);
                taskBean.setAciAccept(0);
                taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                String s = "87CD000800060001006E0002%04X";
                if (taskBean != null) {
                    if (taskBean.getAciIndex() == null) {
                        //还没有提交给aci系统
                        taskBean.setAciAccept(1);
                        amsTaskService.updateAmsTask(taskBean);
                    } else {
                        //已经提交给aci系统
                        s = String.format(s, taskBean.getAciIndex());
                        buffer.put(ByteUtil.hexString2ByteArray(s));
                        buffer.flip();
                        socketChannel.write(buffer);
                        buffer.clear();
                        log.info("cancelOrder: " + s);
                    }

                    if (DateUtil.between(taskBean.getAddtime(), new Date(), DateUnit.MINUTE) > 1) {
                        taskBean.setAciAccept(2);
                        taskBean.setRemark("下发超时1分钟未接收自动取消");
                        amsTaskService.updateAmsTask(taskBean);
                    }
                }

                //充电指令
                taskBean = new AmsTask();
                taskBean.setIsDelete(0);
                taskBean.setAciAccept(0);
                taskBean.setBusinessType(AmsConstant.TASK_BUSINESS_TYPE.TASK_POWER.getValue());
                taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                s = "87CD0008000C000100710008658100010001%04X";
                if (taskBean != null) {

                    s = String.format(s, Integer.parseInt(taskBean.getDeviceName()));
                    buffer.put(ByteUtil.hexString2ByteArray(s));
                    buffer.flip();
                    socketChannel.write(buffer);
                    buffer.clear();
                    log.info("AGV-Power: " + s);
                    taskBean.setServerAck(0);
                    taskBean.setAciAccept(1);
                    taskBean.setIsDelete(null);
                    amsTaskService.updateAmsTask(taskBean);
                }

                //添加指令
                List<AmsTask> taskBeans = amsTaskService.selectTaskNotIssued();

                if (taskBeans != null && taskBeans.size() > 0) {

                    for (AmsTask t : taskBeans) {
                        t.init();
                        List<AmsHexdefineDetail> basTask = amsHexdefineDetailService.selectAmsHexdefineDetailByBusinessType(t.getBusinessType());
                        if (basTask == null || basTask.size() == 0) {
                            t.setIsDelete(1);
                            t.setRemark("该任务类型未定义");
                            log.info("任务类型未定义------" + JSON.toJSONString(t));
                            continue;
                        }

//                        try {
                            String order = amsHexdefineDetailService.concatOrder(t, basTask);
                            log.info("writeOrder:" + order);
                            if (t.getBusinessType().equals("m") ||
                                    t.getBusinessType().equals("01")
                            ) {

                                buffer.put(ByteUtil.hexString2ByteArray(order));
                                buffer.flip();
                                socketChannel.write(buffer);
                                buffer.clear();
                                if (DateUtil.between(t.getAddtime(), new Date(), DateUnit.MINUTE) > 2) {
                                    t.setAciAccept(2);
                                    t.setRoadway("下发超时2分钟未接收自动取消");
                                    amsTaskService.updateAmsTask(t);
                                }
                            } else {
                                buffer.put(ByteUtil.hexString2ByteArray(order));
                                buffer.flip();
                                socketChannel.write(buffer);
                                buffer.clear();
                                // 修改已发送
                                t.setAciAccept(1);
                                t.setRoadway("TS请求反馈任务发送成功！");
                                amsTaskService.updateAmsTask(t);
                            }
                    }
                }
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    throw new BaseException(e.getMessage());
                }

            }catch (Exception e) {
                log.error("NDC通讯异常！");
                try {
                    socketChannel.close();
                } catch (Exception exception) {
                    throw new BaseException(exception.getMessage());
                }
                throw new BaseException(e.getMessage());
            }
        }
    }

    public void readInfo() {
        //读取客户端消息
        new Thread(() -> {
            while (true) {
                try {
                    int readChannels = selector.select();
                    if (readChannels > 0) {
                        Iterator<SelectionKey> iterator = selector.selectedKeys().iterator();
                        while (iterator.hasNext()) {
                            SelectionKey key = iterator.next();
                            if (!key.isReadable()) {
                                iterator.remove();
                                continue;
                            }
                            SocketChannel channel = (SocketChannel) key.channel();
                            ByteBuffer buffer = ByteBuffer.allocate(BUFF_SIZE);
                            channel.read(buffer);
                            List<Order> bufferSeparators = socketBufferService.bufferSeparator(buffer.array());
                            for (Order order : bufferSeparators) {
                                log.info("NDC Msg info is :{}", order);
                                switch (order.getMessageType()) {
                                    case Aci.MSG_ORDER_STATUS://车体状态信息、位置信息
                                        socketBufferService.handleMsgOrderStatus(order.getParameters());
                                        break;
                                    case Aci.MSG_EVENT://任务反馈、自定义消息
                                        socketBufferService.handleMsgEvent(order.getParameters());
                                        break;
                                    case Aci.MSG_STOP://交通管制暂停
                                        socketBufferService.handleMsgStopStart(order.getParameters(), true);
                                        break;
                                    case Aci.MSG_START://启动信息
                                        socketBufferService.handleMsgStopStart(order.getParameters(), false);
                                        break;
                                    case Aci.MSG_ORDER_ACK://信息接收反馈
                                        socketBufferService.handleMsgOrderAck(order.getParameters());
                                        break;
                                }
                            }
                            iterator.remove();
                        }
                    }
//                    Thread.currentThread().sleep(3000);
                } catch (Exception e) {
                    throw new BaseException(e.getMessage());
                }
            }
        }).start();
    }
}
