package com.warewms.ams.ndc.controller;


import com.warewms.ams.ndc.domain.BaseLocationInfo;
import com.warewms.ams.ndc.service.IBaseLocationInfoService;
import com.warewms.common.annotation.Log;
import com.warewms.common.base.controller.BaseController;
import com.warewms.common.base.domain.AjaxResult;
import com.warewms.common.base.page.TableDataInfo;
import com.warewms.common.enums.BusinessType;
import com.warewms.common.utils.poi.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 库位信息Controller
 *
 * @author andy
 * @date 2022-02-18
 */
@RestController
@RequestMapping("/base/locationInfo")
public class BaseLocationInfoController extends BaseController {
    @Autowired
    private IBaseLocationInfoService baseLocationInfoService;

    /**
     * 查询库位信息列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BaseLocationInfo baseLocationInfo) {
        startPage();
        List<BaseLocationInfo> list = baseLocationInfoService.selectBaseLocationInfoList(baseLocationInfo);
        return getDataTable(list);
    }


    /**
     * 获取库位信息详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(baseLocationInfoService.selectBaseLocationInfoById(id));
    }

    /**
     * 修改库位信息
     */
    @Log(title = "库位信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BaseLocationInfo baseLocationInfo) {
        return toAjax(baseLocationInfoService.updateBaseLocationInfo(baseLocationInfo));
    }

    /**
     * 删除库位信息
     */
    @Log(title = "库位信息", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(baseLocationInfoService.deleteBaseLocationInfoByIds(ids));
    }

    /**
     * 获取库位字典
     */
    @GetMapping("/queryDict/{zoneId}")
    public AjaxResult queryDict(@PathVariable("zoneId") Long zoneId) {
        List<BaseLocationInfo> locationInfoList = baseLocationInfoService.selectSortedLocatinListByZoneId(zoneId, 1L,null);
        return AjaxResult.success("", locationInfoList);
    }


    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<BaseLocationInfo> util = new ExcelUtil<>(BaseLocationInfo.class);
        util.importTemplateExcel(response, "库位信息");
    }
}
