package com.warewms.ams.ndc.service;

import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.dto.AmsTaskCancelDTO;
import com.warewms.ams.ndc.dto.AmsTaskDTO;
import com.warewms.ams.ndc.dto.AmsTaskInfoDTO;
import com.warewms.ams.ndc.dto.AmsTaskInsertDTO;
import com.warewms.framework.service.CrudService;

import java.util.List;

/**
 * ams任务列表Service接口
 *
 * @author andy
 * @date 2022-08-08
 */
public interface IAmsTaskService extends CrudService<AmsTask, AmsTaskDTO> {
    /**
     * 查询ams任务列表
     *
     * @param id ams任务列表主键
     * @return ams任务列表
     */
    AmsTask selectAmsTaskById(Long id);

    /**
     * 查询ams任务列表列表
     *
     * @param amsTask ams任务列表
     * @return ams任务列表集合
     */
    List<AmsTask> selectAmsTaskList(AmsTask amsTask);

    /**
     * 查询ams任务列表
     *
     * @param amsTask ams任务列表
     * @return ams任务列表集合
     */
    AmsTask selectAmsTaskByModel(AmsTask amsTask);

    /**
     * 新增ams任务列表
     *
     * @param amsTask ams任务列表
     * @return 结果
     */
    int insertAmsTask(AmsTask amsTask);

    /**
     * 修改ams任务列表
     *
     * @param amsTask ams任务列表
     * @return 结果
     */
    int updateAmsTask(AmsTask amsTask);

    /**
     * 批量删除ams任务列表
     *
     * @param ids 需要删除的ams任务列表主键集合
     * @return 结果
     */
    int deleteAmsTaskByIds(Long[] ids);

    /**
     * 删除ams任务列表信息
     *
     * @param id ams任务列表主键
     * @return 结果
     */
    int deleteAmsTaskById(Long id);

    /**
     * 查询待下发的ams任务
     * @return
     */
    List<AmsTask> selectTaskNotIssued();

    Long amsTaskInsert(AmsTaskInsertDTO amsTaskInsertDTO);

    void ndcAmsTaskInsert(String businessNo, String businessType, Integer stFrom, String stTo);

    AmsTaskInfoDTO getAmsTaskInfo(String businessNo);

    Boolean cancelAmsTask(AmsTaskCancelDTO amsTaskCancelDTO);

    AmsTask getAmsTaskByAciIndex(Integer aciIndex);
}
