package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.warewms.ams.ndc.domain.AmsHexdefineDetail;
import com.warewms.ams.ndc.domain.AmsHexdefineHeader;
import com.warewms.ams.ndc.mapper.AmsHexdefineHeaderMapper;
import com.warewms.ams.ndc.service.IAmsHexdefineHeaderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 16进制指令定义Service业务层处理
 * 
 * @author andy
 * @date 2022-08-12
 */
@Service
public class AmsHexdefineHeaderServiceImpl implements IAmsHexdefineHeaderService
{
    @Autowired
    private AmsHexdefineHeaderMapper amsHexdefineHeaderMapper;

    /**
     * 查询16进制指令定义
     * 
     * @param id 16进制指令定义主键
     * @return 16进制指令定义
     */
    @Override
    public AmsHexdefineHeader selectAmsHexdefineHeaderById(Long id)
    {
        return amsHexdefineHeaderMapper.selectAmsHexdefineHeaderById(id);
    }

    /**
     * 查询16进制指令定义列表
     * 
     * @param amsHexdefineHeader 16进制指令定义
     * @return 16进制指令定义
     */
    @Override
    public List<AmsHexdefineHeader> selectAmsHexdefineHeaderList(AmsHexdefineHeader amsHexdefineHeader)
    {
        return amsHexdefineHeaderMapper.selectAmsHexdefineHeaderList(amsHexdefineHeader);
    }

    /**
     * 查询16进制指令定义
     *
     * @param amsHexdefineHeader 16进制指令定义
     * @return 16进制指令定义
     */
    @Override
    public AmsHexdefineHeader selectAmsHexdefineHeaderByModel(AmsHexdefineHeader amsHexdefineHeader)
    {
        List<AmsHexdefineHeader> list = amsHexdefineHeaderMapper.selectAmsHexdefineHeaderList(amsHexdefineHeader);
        if (list!=null && list.size()>0) {
            return list.get(0);
        } else {
            return null;
        }
    }

    /**
     * 新增16进制指令定义
     * 
     * @param amsHexdefineHeader 16进制指令定义
     * @return 结果
     */
    @Transactional
    @Override
    public int insertAmsHexdefineHeader(AmsHexdefineHeader amsHexdefineHeader)
    {
        int rows = amsHexdefineHeaderMapper.insertAmsHexdefineHeader(amsHexdefineHeader);
        insertAmsHexdefineDetail(amsHexdefineHeader);
        return rows;
    }

    /**
     * 修改16进制指令定义
     * 
     * @param amsHexdefineHeader 16进制指令定义
     * @return 结果
     */
    @Transactional
    @Override
    public int updateAmsHexdefineHeader(AmsHexdefineHeader amsHexdefineHeader)
    {
        amsHexdefineHeaderMapper.deleteAmsHexdefineDetailByBusinessType(amsHexdefineHeader.getId());
        insertAmsHexdefineDetail(amsHexdefineHeader);
        return amsHexdefineHeaderMapper.updateAmsHexdefineHeader(amsHexdefineHeader);
    }

    /**
     * 批量删除16进制指令定义
     * 
     * @param ids 需要删除的16进制指令定义主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteAmsHexdefineHeaderByIds(Long[] ids)
    {
        amsHexdefineHeaderMapper.deleteAmsHexdefineDetailByBusinessTypes(ids);
        return amsHexdefineHeaderMapper.deleteAmsHexdefineHeaderByIds(ids);
    }

    /**
     * 删除16进制指令定义信息
     * 
     * @param id 16进制指令定义主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteAmsHexdefineHeaderById(Long id)
    {
        amsHexdefineHeaderMapper.deleteAmsHexdefineDetailByBusinessType(id);
        return amsHexdefineHeaderMapper.deleteAmsHexdefineHeaderById(id);
    }

    /**
     * 新增任务定义（lp0-lp30）信息
     * 
     * @param amsHexdefineHeader 16进制指令定义对象
     */
    public void insertAmsHexdefineDetail(AmsHexdefineHeader amsHexdefineHeader)
    {
        List<AmsHexdefineDetail> amsHexdefineDetailList = amsHexdefineHeader.getAmsHexdefineDetailList();
        String businessType = amsHexdefineHeader.getBusinessType();
        if (CollectionUtil.isNotEmpty(amsHexdefineDetailList))
        {
            List<AmsHexdefineDetail> list = new ArrayList<AmsHexdefineDetail>();
            for (AmsHexdefineDetail amsHexdefineDetail : amsHexdefineDetailList)
            {
                amsHexdefineDetail.setBusinessType(businessType);
                list.add(amsHexdefineDetail);
            }
            if (list.size() > 0)
            {
                amsHexdefineHeaderMapper.batchAmsHexdefineDetail(list);
            }
        }
    }
}
