package com.warewms.ams.ndc.service.impl;

import com.warewms.ams.ndc.domain.AmsNdcEvent;
import com.warewms.ams.ndc.mapper.AmsNdcEventMapper;
import com.warewms.ams.ndc.service.IAmsNdcEventService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Ndc反馈事件代码字典Service业务层处理
 * 
 * @author andy
 * @date 2022-08-08
 */
@Service
public class AmsNdcEventServiceImpl implements IAmsNdcEventService
{
    @Autowired
    private AmsNdcEventMapper amsNdcEventMapper;

    /**
     * 查询Ndc反馈事件代码字典
     * 
     * @param id Ndc反馈事件代码字典主键
     * @return Ndc反馈事件代码字典
     */
    @Override
    public AmsNdcEvent selectAmsNdcEventById(Long id)
    {
        return amsNdcEventMapper.selectAmsNdcEventById(id);
    }

    /**
     * 查询Ndc反馈事件代码字典列表
     * 
     * @param amsNdcEvent Ndc反馈事件代码字典
     * @return Ndc反馈事件代码字典
     */
    @Override
    public List<AmsNdcEvent> selectAmsNdcEventList(AmsNdcEvent amsNdcEvent)
    {
        return amsNdcEventMapper.selectAmsNdcEventList(amsNdcEvent);
    }

    /**
     * 查询Ndc反馈事件代码字典
     *
     * @param amsNdcEvent Ndc反馈事件代码字典
     * @return Ndc反馈事件代码字典
     */
    @Override
    public AmsNdcEvent selectAmsNdcEventByModel(AmsNdcEvent amsNdcEvent)
    {
        List<AmsNdcEvent> list = amsNdcEventMapper.selectAmsNdcEventList(amsNdcEvent);
        if (list!=null && list.size()>0) {
            return list.get(0);
        } else {
            return null;
        }
    }

    /**
     * 新增Ndc反馈事件代码字典
     * 
     * @param amsNdcEvent Ndc反馈事件代码字典
     * @return 结果
     */
    @Override
    public int insertAmsNdcEvent(AmsNdcEvent amsNdcEvent)
    {
        return amsNdcEventMapper.insertAmsNdcEvent(amsNdcEvent);
    }

    /**
     * 修改Ndc反馈事件代码字典
     * 
     * @param amsNdcEvent Ndc反馈事件代码字典
     * @return 结果
     */
    @Override
    public int updateAmsNdcEvent(AmsNdcEvent amsNdcEvent)
    {
        return amsNdcEventMapper.updateAmsNdcEvent(amsNdcEvent);
    }

    /**
     * 批量删除Ndc反馈事件代码字典
     * 
     * @param ids 需要删除的Ndc反馈事件代码字典主键
     * @return 结果
     */
    @Override
    public int deleteAmsNdcEventByIds(Long[] ids)
    {
        return amsNdcEventMapper.deleteAmsNdcEventByIds(ids);
    }

    /**
     * 删除Ndc反馈事件代码字典信息
     * 
     * @param id Ndc反馈事件代码字典主键
     * @return 结果
     */
    @Override
    public int deleteAmsNdcEventById(Long id)
    {
        return amsNdcEventMapper.deleteAmsNdcEventById(id);
    }
}
