package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.domain.BaseLocationInfo;
import com.warewms.ams.ndc.mapper.BaseLocationInfoMapper;
import com.warewms.ams.ndc.service.IBaseLocationInfoService;
import com.warewms.common.utils.redis.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;

/**
 * 库位信息Service业务层处理
 *
 * @author andy
 * @date 2022-02-18
 */
@Slf4j
@Service
public class BaseLocationInfoServiceImpl implements IBaseLocationInfoService {

    @Autowired
    private BaseLocationInfoMapper baseLocationInfoMapper;

    @Autowired
    private RedisCache redisCache;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init() {
        loadingLocationCache();
    }

    /**
     * 加载参数缓存数据
     */
    public void loadingLocationCache() {
        List<BaseLocationInfo> baseLocationInfoList = baseLocationInfoMapper.selectList(Wrappers.emptyWrapper());
        for (BaseLocationInfo baseLocationInfo : baseLocationInfoList) {
            redisCache.setCacheObject(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(baseLocationInfo.getLocationNo()), baseLocationInfo.getAgvStation());
            redisCache.setCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(baseLocationInfo.getLocationNo()), baseLocationInfo.getRowNo());
        }
    }

    @Override
    public List<BaseLocationInfo> queryBaseLocationInfoList(Integer zoneId, String rowNo) {
        return queryBaseLocationInfoList(zoneId, null, rowNo);
    }

    private List<BaseLocationInfo> queryBaseLocationInfoList(Integer zoneId, String locationNo, String rowNo) {
        return baseLocationInfoMapper.selectList(Wrappers.<BaseLocationInfo>lambdaQuery()
                .eq(ObjectUtil.isNotNull(zoneId), BaseLocationInfo::getZoneId, zoneId)
                .eq(StringUtils.isNotBlank(locationNo), BaseLocationInfo::getLocationNo, locationNo)
                .eq(StringUtils.isNotBlank(rowNo), BaseLocationInfo::getRowNo, rowNo));
    }

    /**
     * 查询库位信息
     *
     * @param id 库位信息主键
     * @return 库位信息
     */
    @Override
    public BaseLocationInfo selectBaseLocationInfoById(Long id) {
        return baseLocationInfoMapper.selectBaseLocationInfoById(id);
    }

    /**
     * 查询库位信息列表
     *
     * @param baseLocationInfo 库位信息
     * @return 库位信息
     */
    @Override
    public List<BaseLocationInfo> selectBaseLocationInfoList(BaseLocationInfo baseLocationInfo) {
        return baseLocationInfoMapper.selectBaseLocationInfoList(baseLocationInfo);
    }

    /**
     * 修改库位信息
     *
     * @param baseLocationInfo 库位信息
     * @return 结果
     */
    @Override
    public int updateBaseLocationInfo(BaseLocationInfo baseLocationInfo) {
        return baseLocationInfoMapper.updateBaseLocationInfo(baseLocationInfo);
    }

    /**
     * 批量删除库位信息
     *
     * @param ids 需要删除的库位信息主键
     * @return 结果
     */
    @Override
    public int deleteBaseLocationInfoByIds(Long[] ids) {
        return baseLocationInfoMapper.deleteBaseLocationInfoByIds(ids);
    }

    @Override
    public List<BaseLocationInfo> selectSortedLocatinListByZoneId(Long zoneId, Long warehouseId,String orderBy) {
        BaseLocationInfo query = new BaseLocationInfo();
        query.setWarehouseId(warehouseId);
        query.setZoneId(zoneId);
        return baseLocationInfoMapper.selectSortedLocationListByZoneId(query);
    }

}
