package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.common.Aci;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.ams.ndc.domain.AmsNdcEvent;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.entity.Order;
import com.warewms.ams.ndc.service.IAmsNdcEventService;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.ams.ndc.service.SocketBufferService;
import com.warewms.ams.ndc.service.StatusUpdateService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Created by IntelliJ IDEA.
 * User: andy.qu
 * Date: 2022/8/5
 * Description:
 */
@Slf4j
@Component
public class SocketBufferServiceImpl implements SocketBufferService {

    @Autowired
    private IAmsNdcEventService amsNdcEventService;
    @Autowired
    private IAmsTaskService amsTaskService;
    @Autowired
    private StatusUpdateService statusUpdateService;

    // 创建一个线程池
    private static ExecutorService executorService;

    static {
        // 创建一个线程池
        executorService = new ThreadPoolExecutor(5, 30
                , 60L, TimeUnit.SECONDS
                , new SynchronousQueue<Runnable>(true));
    }

    @Override
    public List<Order> bufferSeparator(byte[] buffer) {
        List<Order> orderList = new ArrayList<>();
        String bufferStr = ByteUtil.byteArray2HexString(buffer);
        //通过固定的头部 87CD0008 将多条消息拆分开
        String[] signalList = bufferStr.split("87CD0008");
        for (String signalStr : signalList) {

            if (StringUtils.isNotEmpty(signalStr)) {

                byte[] hexData = ByteUtil.hexString2ByteArray(signalStr);
                //消息体短缺
                if (hexData.length < 8) continue;

                Order receiveOrder = new Order();
                receiveOrder.setHexData(hexData);

                //<<function code>>
                int functionCode = ByteUtil.getUnsignedShort(hexData, 2);
                receiveOrder.setFunctionCode(functionCode);

                //<<message type>>
                String messageType = ByteUtil.byteArray2String(new byte[]{hexData[5]});
                receiveOrder.setMessageType(messageType);

                //<<number of parameter>> 目前观察下来都是50
                int paramNum = ByteUtil.getUnsignedShort(hexData, 6);
                receiveOrder.setParameterNums(paramNum);

                //截取size of parameter之后的内容
                String paramStr = signalStr.substring(16);
                byte[] parameters = ByteUtil.hexString2ByteArray(paramStr);
                //参数长度异常（被截取 || 冗余带另外一条不完整的数据）
                //冗余数据：m消息的ack parameters实际长度是6，但是长度定义的是5||8。针对此项目写死长度，6是m消息的ack。8是下发任务的ack
//                if (null == parameters || parameters.length != paramNum) continue;
                if (null == parameters) continue;
                if (!Aci.MSG_ORDER_ACK.equals(messageType) && parameters.length != paramNum) continue;
                if (Aci.MSG_ORDER_ACK.equals(messageType) && (parameters.length != 6 && parameters.length != 8))
                    continue;
                receiveOrder.setParameters(parameters);

                orderList.add(receiveOrder);
            }
        }

        return orderList;
    }

    @Override
    public void handleMsgOrderStatus(byte[] parameters) {
        int carNo = ByteUtil.getUnsignedShort(parameters, 6);
        switch (parameters[5]) {
            case 0x58:
                //电池余量为千分比
                //car.setBattery(ByteUtil.getUnsignedShort(parameters, 14) / 10);
                //car.update();
                break;
            case 0x63:
                //xy轴位置
                //car.setX(251770 - ByteUtil.getUnsignedShort(parameters, 6) * 10);
                //car.setY(96620 - ByteUtil.getUnsignedShort(parameters, 14) * 10);
                //car.update();
                break;
        }
    }

    @Override
    public void handleMsgEvent(byte[] parameters) {
        int icarNo = ByteUtil.getShort(parameters, 40);
        AmsNdcEvent event = new AmsNdcEvent();
        event.setType(parameters[13] & 0xff);
        event.setEcode((int) ByteUtil.getShort(parameters, 16));
        AmsNdcEvent eventBean = amsNdcEventService.selectAmsNdcEventByModel(event);
        if (ObjectUtil.isNull(eventBean)) return;
        event = eventBean;
        event.setLocationindex(ByteUtil.getInt(parameters, 26));
        event.setLp1("" + ByteUtil.getInt(parameters, 34));
        log.info("lp1 is :{}", event.getLp1());
        event.setLp2("" + ByteUtil.getInt(parameters, 38));
        log.info("lp2 is :{}", event.getLp2());
        event.setLp3("" + ByteUtil.getInt(parameters, 42));
        log.info("lp3 is :{}", event.getLp3());
        event.setLp4("" + ByteUtil.getInt(parameters, 46));
        int eventId = event.getId();
        log.info("evenId is :{}", eventId);
        if (ObjectUtil.isNotNull(AmsConstant.TASK_STS.getValueByKey(eventId))) {
            statusUpdateService.taskStateResponse(AmsConstant.TASK_STS.getValueByKey(eventId), icarNo, Integer.parseInt(event.getLp1()));
        }
        AmsTask taskBean = new AmsTask();

//            log.info("----event:" + eventId);
            switch (eventId) {
                case 85://user event ts cancel                  任务取消

                    log.info("任务回调执行取消:" + JSON.toJSONString(event));
                    taskBean.setAciIndex(Integer.parseInt(event.getLp1()));
                    taskBean = amsTaskService.selectAmsTaskByModel(taskBean);

                    AmsTask taskBeanUpdate = new AmsTask();
                    taskBeanUpdate.setId(taskBean.getId());
                    taskBeanUpdate.setEvent(eventId);
                    taskBeanUpdate.setServerAck(0);
                    taskBeanUpdate.setAciAccept(1);
                    taskBeanUpdate.setRoadway("任务回调执行取消");
                    taskBeanUpdate.setSystemStatus(null);
                    taskBeanUpdate.setIsDelete(null);
                    amsTaskService.updateAmsTask(taskBeanUpdate);
                    //任务状态回调
                    long taskId_85 = taskBean.getId().longValue();
                    executorService.execute(() -> {
                        statusUpdateService.updateStatus(taskId_85, icarNo);
                    });
                    break;
                case 86://user event Order Receive              任务开始
                case 87://user event FETCH(LP1) is Invalid!!
                case 88://user event FETCH Covert Failed
                case 89://userevent DELIV Convert Failed
                case 90://user event Order Complete             任务完成
                case 91://user event Loaded                     取货完成
                case 92://user event Delivered                  卸货完成
                case 93://user event Redirect message for FETCH
                case 94://user event Redirect message for DELIVER
                case 113://user event Fetch Empty
                    log.info("----event:" + eventId + ":" + JSON.toJSONString(event));

                    taskBean.setAciIndex(Integer.parseInt(event.getLp1()));
                    taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                    if (taskBean != null) {
                        AmsTask taskBeanUpdate01 = new AmsTask();
                        taskBeanUpdate01.setId(taskBean.getId());
                        taskBeanUpdate01.setEvent(eventId);
                        taskBeanUpdate01.setServerAck(0);
                        taskBeanUpdate01.setSystemStatus(null);
                        taskBeanUpdate01.setIsDelete(null);
                        amsTaskService.updateAmsTask(taskBeanUpdate01);

                        long taskId = taskBean.getId().longValue();
                        executorService.execute(() -> {
                            statusUpdateService.updateStatus(taskId, icarNo);
                        });
                    }
                    break;
                case 112://user event  Carrier status
                    //carStatus.setSpeed(ByteUtil.getInt(buffer, 34));
                    //carStatus.setErr(ByteUtil.getInt(buffer, 38));
                    //carStatus.setOnload(ByteUtil.getInt(buffer, 42));
                    //carStatus.setOncharge(ByteUtil.getInt(buffer, 46));
                    break;
                case 114:
                    //60任务卸货确认
                    Integer aciIndex = Integer.parseInt(event.getLp1());
                    log.info("MSG_EVENT114:卸货确认{}", JSON.toJSONString(event));

                    AmsTask task60Search = new AmsTask();
                    task60Search.setAciIndex(aciIndex);
                    AmsTask task60 = amsTaskService.selectAmsTaskByModel(task60Search);

                    if (task60 == null) {
                        log.info("aciIndex：" + aciIndex + "查询不到对应的60任务");
                    } else {
                        //解析卸货确认状态
                        // agvUnloadingConfirmService.confirmAnalysis(task60.getTaskNo(), aciIndex, "unload", icarNo);
                    }
                    break;
                case 115:
                    //60任务取货确认
                    Integer aciIndexPick = Integer.parseInt(event.getLp1());
                    log.info("MSG_EVENT114:取货确认{}", JSON.toJSONString(event));

                    AmsTask task60SearchPick = new AmsTask();
                    task60SearchPick.setAciIndex(aciIndexPick);
                    AmsTask task60Pick = amsTaskService.selectAmsTaskByModel(task60SearchPick);

                    if (task60Pick == null) {
                        log.info("aciIndex：" + aciIndexPick + "查询不到对应的60任务");
                    } else {
                        //解析取货确认状态
                        // agvUnloadingConfirmService.confirmAnalysis(task60Pick.getTaskNo(), aciIndexPick, "pickup", icarNo);
                    }
                    break;
                case 122:
                    //车辆暂停信息
                    try {
                        log.info("carNo:" + icarNo);
                        String msg = "";
                        if (event.getLp2().equals("1")) {
                            msg = "暂停";
                        } else if (event.getLp2().equals("0")) {
                            msg = "启动";
                        }
                        // TODO 车辆暂停上报
                    } catch (Exception e) {
                        log.error("车辆暂停反馈错误", e);
                    }
                    break;
                case 123:
                    //空托盘堆叠层数申请
                    log.info("空托盘堆叠层数申请");
                    Integer index130 = Integer.parseInt(event.getLp1());
                    AmsTask task130 = new AmsTask();
                    task130.setAciIndex(index130);
                    task130 = amsTaskService.selectAmsTaskByModel(task130);
                    if (task130 != null) {
                        // agvUnloadingConfirmService.confirmEmptyPlate(task130.getTaskNo());
                    }
                    break;
                case 134:
                    break;
                case 135:
                    //stopword异常
                    Integer index135 = Integer.parseInt(event.getLp2());
                    log.info("Stopword Error:" + JSON.toJSONString(event));

                    AmsTask task135 = new AmsTask();
                    task135.setAciIndex(index135);
                    task135 = amsTaskService.selectAmsTaskByModel(task135);
                    // agvUnloadingConfirmService.agvError(event,task135);
                    break;
                case 136:
                    //failcode异常
                    Integer index136 = Integer.parseInt(event.getLp2());
                    log.info("Failcode Error:" + JSON.toJSONString(event));

                    AmsTask task136 = new AmsTask();
                    task136.setAciIndex(index136);
                    task136 = amsTaskService.selectAmsTaskByModel(task136);
                    // agvUnloadingConfirmService.agvError(event,task136);
                    break;
                case 137:
                    //stopword2异常
                    Integer index137 = Integer.parseInt(event.getLp2());
                    log.info("Stopword2 Error:" + JSON.toJSONString(event));

                    AmsTask task137 = new AmsTask();
                    task137.setAciIndex(index137);
                    task137 = amsTaskService.selectAmsTaskByModel(task137);
                    // agvUnloadingConfirmService.agvError(event,task137);
                    break;
                case 139:
                case 140:
                    //AGV其他异常上报
                    Integer index139 = Integer.parseInt(event.getLp2());
                    log.info("Safety Laser Trigger:" + JSON.toJSONString(event));

                    AmsTask task139 = new AmsTask();
                    task139.setAciIndex(index139);
                    task139 = amsTaskService.selectAmsTaskByModel(task139);
                    // agvUnloadingConfirmService.agvError(event,task139);
                    break;
                default:
                    log.info("----event:" + eventId);
                    break;
            }

    }

    @Override
    public void handleMsgOrderAck(byte[] parameters) {
        AmsTask taskBean = new AmsTask();
        String ndcStatusValue;
        switch (parameters[3] & 0xff) {//发送指令后返回，需根据ikey确定index或执行结果
            case 0://拒接
            case 1://order acknowledge 接收
                ndcStatusValue = "Order acknowledge, input in queue || Order Rejected.";
                taskBean.setIkey((long) ByteUtil.getUnsignedShort(parameters, 6));
                taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                if (taskBean != null) {

                    if (parameters[3] == 1 || parameters[3] == 27) {
                        taskBean.setAciAccept(1);
                    }
                    taskBean.setAciIndex(parameters[1] & 0xff);
                    taskBean.setSystemStatus((parameters[3] & 0xff));
                    taskBean.setServerAck(0);
                    taskBean.setIsDelete(null);
                    taskBean.setRoadway(ndcStatusValue);
                    amsTaskService.updateAmsTask(taskBean);

                    log.info("任务下发完成计数" + JSON.toJSONString(taskBean));
                    log.info("update ack:" + JSON.toJSONString(taskBean));
                } else {

                    log.error("任务下发完成ack未成功找到对应amsTask：" + ByteUtil.getUnsignedShort(parameters, 6));
                }
                break;
            case 7:

                ndcStatusValue = "Parameter acknowledge(<m>msg)";
                //修改参数确认反馈（m消息）
                taskBean.setAciIndex(ByteUtil.getUnsignedShort(parameters, 0));
                taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                if (taskBean != null) {
                    AmsTask taskM = new AmsTask();
                    taskM.setRemark(taskBean.getTaskNo());
                    taskM = amsTaskService.selectAmsTaskByModel(taskBean);
                    if (taskM != null) {
                        if (parameters[3] == 7 || parameters[4] == Integer.parseInt(taskM.getExt2())) {
                            taskM.setAciAccept(1);
                            log.info("m消息ack成功：" + JSON.toJSONString(taskBean));
                        } else {

                            log.error("m消息ack失败，参数3：" + parameters[7] + "，参数4：" + parameters[4] + "，taskBean：" + JSON.toJSONString(taskBean));
                        }
                        taskM.setSystemStatus((parameters[3] & 0xff));
                        taskM.setServerAck(0);
                        taskM.setIsDelete(null);
                        taskM.setRoadway(ndcStatusValue);
                        amsTaskService.updateAmsTask(taskM);
                    } else {

                        log.error("m消息ack失败，未找到对应的AmsTask1：" + taskBean.getTaskNo());
                    }
                } else {

                    log.error("m消息ack失败，未找到对应的AmsTask，index：" + ByteUtil.getUnsignedShort(parameters, 0));
                }
                break;
            case 8:
            case 9:
            case 10:
            case 11:
            case 15:
            case 16:
            case 26:
            case 27:
            case 28:

                ndcStatusValue = "Error Value:[" + (parameters[3] & 0xff) + "]";
                taskBean.setIkey((long) ByteUtil.getUnsignedShort(parameters, 6));
                taskBean = amsTaskService.selectAmsTaskByModel(taskBean);
                log.info("任务下发接收反馈异常," + ndcStatusValue);
                if (taskBean != null) {

                    taskBean.setAciAccept(1);
                    taskBean.setSystemStatus((parameters[3] & 0xff));
                    taskBean.setServerAck(0);
                    taskBean.setIsDelete(null);
                    taskBean.setRoadway(ndcStatusValue);
                    amsTaskService.updateAmsTask(taskBean);

                    log.info("任务下发接收反馈异常码:" + JSON.toJSONString(taskBean));
                }
                break;
            case 4:
            case 25:

                //删除任务确认反馈（m消息）
                ndcStatusValue = "Order Cancelled || Cancel Acknowledge.";
                AmsTask deleteTask = new AmsTask();
                deleteTask.setAciIndex(ByteUtil.getUnsignedShort(parameters, 0));
                log.info("删除任务M消息确认{}", deleteTask.getAciIndex());
                deleteTask = amsTaskService.selectAmsTaskByModel(deleteTask);
                if (deleteTask != null) {
                    deleteTask.setAciAccept(1);
                    deleteTask.setRoadway(ndcStatusValue);
                    amsTaskService.updateAmsTask(deleteTask);
                }
                break;
            default:

                ndcStatusValue = "Unknow Value:[" + (parameters[3] & 0xff) + "]";
                AmsTask unknowTask = new AmsTask();
                unknowTask.setAciIndex(ByteUtil.getUnsignedShort(parameters, 0));
                log.info("消息接收反馈未匹配对应Value：{}", unknowTask.getAciIndex());
                unknowTask = amsTaskService.selectAmsTaskByModel(unknowTask);
                if (unknowTask != null) {

                    unknowTask.setAciAccept(1);
                    unknowTask.setIsDelete(null);
                    unknowTask.setRoadway(ndcStatusValue);
                    amsTaskService.updateAmsTask(unknowTask);
                    log.info("消息接收反馈未匹配对应Value, " + JSON.toJSONString(unknowTask));
                } else {

                    unknowTask = new AmsTask();
                    unknowTask.setIkey((long) ByteUtil.getUnsignedShort(parameters, 6));
                    unknowTask = amsTaskService.selectAmsTaskByModel(unknowTask);
                    if (unknowTask != null) {

                        unknowTask.setAciAccept(1);
                        unknowTask.setIsDelete(null);
                        unknowTask.setRoadway(ndcStatusValue);
                        amsTaskService.updateAmsTask(unknowTask);
                        log.info("消息接收反馈未匹配对应Value, " + JSON.toJSONString(unknowTask));
                    }
                }
                break;
        }
    }

    @Override
    public void handleMsgStopStart(byte[] parameters, boolean isStop) {

    }
}
