package com.warewms.ams.nio;

import cn.hutool.core.util.NumberUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Iterator;

@Slf4j
public abstract class NioBaseServer {

    private ServerSocketChannel listenChannel;

    private Selector selector;

    public NioBaseServer(int port) {
        try {
            selector = Selector.open();
            listenChannel = ServerSocketChannel.open();
            listenChannel.socket().bind(new InetSocketAddress(port));
            listenChannel.configureBlocking(Boolean.FALSE);
            listenChannel.register(selector, SelectionKey.OP_ACCEPT);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void start() {
        try {
            while (true) {
                int count = selector.select();
                if (NumberUtil.isGreater(BigDecimal.valueOf(count), BigDecimal.ZERO)) {
                    Iterator<SelectionKey> selectionKeyIterator = selector.selectedKeys().iterator();
                    while (selectionKeyIterator.hasNext()) {
                        SelectionKey selectionKey = selectionKeyIterator.next();
                        //连接事件
                        if (selectionKey.isAcceptable()) {
                            SocketChannel socketChannel = listenChannel.accept();
                            socketChannel.configureBlocking(Boolean.FALSE);
                            socketChannel.register(selector, SelectionKey.OP_READ);
                            log.info("The server has a new client connection :{}", socketChannel.getRemoteAddress());
                        }
                        //读取事件
                        if (selectionKey.isReadable()) read(selectionKey);
                        selectionKeyIterator.remove();
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //读取客户端消息
    private void read(SelectionKey key) {
        //取到关联的channel
        SocketChannel channel = null;
        try {
            //得到channel
            channel = (SocketChannel) key.channel();
            channel.configureBlocking(Boolean.FALSE);
            //创建buffer
            ByteBuffer buffer = ByteBuffer.allocate(1024);
            int count = channel.read(buffer);
            if (count > 0) {
                readData(buffer.array());
            }
        } catch (IOException e) {
            try {
                log.info("the client {} connection is disconnected", channel.getRemoteAddress());
                //取消注册
                key.cancel();
                //关闭通道
                channel.close();
            } catch (IOException e2) {
                e2.printStackTrace();
            }
        }
    }


    protected abstract void readData(byte[] data);

}
