package com.warewms.ams.thread;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.ams.ndc.common.CRC16Util;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.common.utils.redis.RedisCache;
import com.warewms.common.utils.spring.SpringUtils;
import com.warewms.system.entity.SysConfig;
import com.warewms.system.service.ISysConfigService;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.Date;

/**
 * 客户端接入监听类
 *
 * @Author: five
 * @Date: 2019/12/23
 */
@Slf4j
public class ClientHandler {

    /**
     * 数据连接值
     */
    public static final int MAX_DATA_LEN = 5;
    private final Socket socket;
    private ISysConfigService configService;

    private IAmsTaskService amsTaskService;

    private RedisCache redisCache;



    public ClientHandler(Socket socket, ISysConfigService configService) {
        this.socket = socket;
        this.configService = configService;
        this.redisCache = SpringUtils.getBean("redisCache");
        this.amsTaskService = SpringUtils.getBean("amsTaskServiceImpl");

    }

    public void start() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                doStart();
            }
        }).start();
    }

    /**
     * 对客户端的业务处理，接收并重写回去
     */
    private void doStart() {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            inputStream = socket.getInputStream();
            outputStream = socket.getOutputStream();
            while (true) {
                byte[] data = new byte[MAX_DATA_LEN];
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                // 服务器向客户端写，如果客户端断开了，这边会抛出异常，释放线程
                outputStream.write(ByteUtil.hexString2ByteArray("0000000000"));
                //根据数据判断是否超高，然后变更字段信息
                while ((inputStream.read(data)) != -1) {

//                    log.info("------------光栅检测结果 > " + ByteUtil.byteArray2HexString(data));
                    SysConfig sysConfig = new SysConfig();
                    switch (ByteUtil.byteArray2HexString(data)) {
                        case "3F00110D0A":
                            //光栅1 超高Y
                            log.info("------------光栅1 超高Y------------>");
                            sysConfig.setConfigId(Long.valueOf("6"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("Y");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00100D0A":
                            //光栅1 不超高N
//                            log.info("------------光栅1 不超高N------------>");
                            sysConfig.setConfigId(Long.valueOf("6"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("N");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00210D0A":
                            //光栅2 超高Y
                            log.info("------------光栅2 超高Y------------>");
                            sysConfig.setConfigId(Long.valueOf("7"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("Y");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00200D0A":
                            //光栅2 不超高N
//                            log.info("------------光栅2 不超高N------------>");
                            sysConfig.setConfigId(Long.valueOf("7"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("N");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00310D0A":
                            //光栅3 超高Y
                            log.info("------------光栅3 超高Y------------>");
                            sysConfig.setConfigId(Long.valueOf("8"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("Y");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00300D0A":
                            //光栅3 不超高N
//                            log.info("------------光栅3 不超高N------------>");
                            sysConfig.setConfigId(Long.valueOf("8"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("N");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00410D0A":
                            //光栅4 超高Y
                            log.info("------------光栅4 超高Y------------>");
                            sysConfig.setConfigId(Long.valueOf("9"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("Y");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00400D0A":
                            //光栅4 不超高N
//                            log.info("------------光栅4 不超高N------------>");
                            sysConfig.setConfigId(Long.valueOf("9"));
                            sysConfig.setConfigValue("N");
                            sysConfig.setConfigType("N");
                            sysConfig.setUpdateTime(new Date());
                            configService.updateConfig(sysConfig);
                            break;
                        case "3F00500D0A":
                            //入库区域光栅检测到信号
                            if (redisCache.checkIsExist(AmsConstant.AREA_INFO.STORAGE_AREA.getKey())) break;
                            insertAmsTask(AmsConstant.AREA_INFO.STORAGE_AREA.getValue());
                            redisCache.setCacheObject(AmsConstant.AREA_INFO.STORAGE_AREA.getKey(), Boolean.TRUE);
                            break;
                        case "3F00600D0A":
                            //出库区域光栅检测到信号
                            if (redisCache.checkIsExist(AmsConstant.AREA_INFO.EXIT_AREA.getKey())) break;
                            insertAmsTask(AmsConstant.AREA_INFO.EXIT_AREA.getValue());
                            redisCache.setCacheObject(AmsConstant.AREA_INFO.EXIT_AREA.getKey(), Boolean.TRUE);
                            break;
                    }
                }
            }
        } catch (IOException e) {
            log.info("光栅服务器读,线程资源释放：" + socket.getInetAddress() + ",p:" + socket.getPort());
        } finally {
            try {
                inputStream.close();
                outputStream.close();
                socket.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void insertAmsTask(String op){
        AmsTask amsTask = new AmsTask();
        amsTask.setTaskNo(IdWorker.getIdStr());
        amsTask.setIkey(Long.parseLong(String.valueOf(CRC16Util.calcCrc16(ByteUtil.string2byteArray(amsTask.getTaskNo())))));
        amsTask.setStFrom(Integer.valueOf(AmsConstant.LOC_MIDDLE_CACHE.toString()));
        amsTask.setStTo(Integer.valueOf(AmsConstant.LOC_SORTATION_CACHE.toString()));
        amsTask.setPriority(1);
        amsTask.setBusinessType(AmsConstant.TASK_BUSINESS_TYPE.TASK_76.getValue());
        amsTask.setExt1(op);
        amsTaskService.insertAmsTask(amsTask);
    }

}
