package com.warewms.ams.thread;

import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.common.utils.redis.RedisCache;
import com.warewms.common.utils.spring.SpringUtils;
import com.warewms.system.entity.SysConfig;
import com.warewms.system.service.ISysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * Server监听端口主服务
 *
 * @Author: five
 * @Date: 2019/12/23
 */
@Slf4j
public class ServerGs {

    private ServerSocket serverSocket;

    private ISysConfigService configService;

    private ScheduledExecutorService executor;

    private RedisCache redisCache;

    /**
     * 监听端口
     *
     * @param port
     */
    public ServerGs(int port, ISysConfigService configService) {
        try {
            this.serverSocket = new ServerSocket(port);
            this.configService = configService;
            this.executor = SpringUtils.getBean("scheduledExecutorService");
            this.redisCache = SpringUtils.getBean("redisCache");
            System.out.println("光栅服务端启动成功，端口：" + port);
        } catch (IOException e) {
            System.out.println("光栅服务端启动失败");
            if (serverSocket != null) {
                try {
                    serverSocket.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * 线程模式，防止注释
     */
    public void start() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                doStart();
            }
        }).start();
    }

    /**
     * 启动客户端接受
     */
    private void doStart() {
        while (true) {
            try {
                Socket client = serverSocket.accept();
                log.info("----->有光栅客户端新连接:ip," + client.getInetAddress() + ",p:" + client.getPort());
                executor.scheduleWithFixedDelay(() -> {
                    OutputStream os = null;
                    try {
                        os = client.getOutputStream();

                        //根据参数判定哪个光栅进行超高检测  > condifvalue为Y开启光栅检测，N关闭光栅检测
                        SysConfig config = new SysConfig();
                        config.setConfigName("光栅");
                        config.setConfigValue("Y");
                        List<SysConfig> list = configService.selectConfigList(config);
                        for (SysConfig sysConfig : list) {
                            String detectionSignal = AmsConstant.GRATING_DETECTION_SIGNAL.getValueByKey(sysConfig.getConfigKey());
                            if (StringUtils.isBlank(detectionSignal)) continue;
                            os.write(ByteUtil.hexString2ByteArray(detectionSignal));
                            os.flush();
                            Thread.sleep(200);
                        }
                    } catch (Exception e) {
                        log.error("光栅客户端断开：" + e.getMessage());
                        //关闭连接
                        try {
                            os.close();
                            client.close();
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                        throw new RuntimeException("光栅客户端断开,释放线程资源：" + e.getMessage());
                    }
                }, 1, 1, TimeUnit.SECONDS);
                new ClientHandler(client, configService).start();
            } catch (IOException e) {
                System.out.println("光栅服务端异常:" + e);
            }
        }
    }
}
