package com.warewms.ams.wcs.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.ams.ndc.common.CRC16Util;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.ams.wcs.form.BtnOpForm;
import com.warewms.ams.wcs.service.IWcsService;
import com.warewms.common.utils.redis.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 出库单头Service业务层处理
 *
 * @author ruoyi
 * @date 2022-10-18
 */
@Slf4j
@Service
public class WcsServiceImpl implements IWcsService {


    @Autowired
    private RedisCache redisCache;
    @Autowired
    private IAmsTaskService amsTaskService;

    /**
     * 按钮盒关闭交通管制
     */
    private final static String OPEN = "OPEN";

    /**
     * 按钮盒开启交通管制
     */
    private final static String CLOSE = "CLOSE";

    /**
     * 按钮盒对应出库区域ip(旧)
     */
    private final static String EXIT_AREA_IP = "192.168.77.105";

    /**
     * 按钮盒对应出库区域ip
     */
    private final static String NEW_EXIT_AREA_IP = "192.168.77.108";

    /**
     * 按钮盒对应入库区域ip(旧)
     */
    private final static String STORAGE_AREA_IP = "192.168.77.104";

    /**
     * 按钮盒对应入库区域ip
     */
    private final static String NEW_STORAGE_AREA_IP = "192.168.77.107";

    /**
     * 按钮盒控制对应NDC任务类型
     */
    private static final Map<String, String> buttonBoxControlMap= new HashMap<String, String>(){
        {
            put(OPEN, AmsConstant.TASK_BUSINESS_TYPE.TASK_75.getValue());
            put(CLOSE, AmsConstant.TASK_BUSINESS_TYPE.TASK_74.getValue());
        }
    };

    /**
     * 按钮盒解除管控对应光栅缓存key
     */
    private static final Map<String, String> buttonBoxGratingRedisKeyMap= new HashMap<String, String>(){
        {
            put(AmsConstant.AREA_INFO.STORAGE_AREA.getValue(), AmsConstant.GRATING_RELIEVE_CMD.STORAGE_GRATING.getKey());
            put(AmsConstant.AREA_INFO.EXIT_AREA.getValue(), AmsConstant.GRATING_RELIEVE_CMD.EXIT_GRATING.getKey());
        }
    };

    /**
     * 按钮盒区域对应NDCid
     */
    private static final Map<String, String> buttonBoxAreaMap= new HashMap<String, String>(){
        {
            put(EXIT_AREA_IP, AmsConstant.AREA_INFO.EXIT_AREA.getValue());
            put(STORAGE_AREA_IP, AmsConstant.AREA_INFO.STORAGE_AREA.getValue());
            put(NEW_EXIT_AREA_IP, AmsConstant.AREA_INFO.EXIT_AREA.getValue());
            put(NEW_STORAGE_AREA_IP, AmsConstant.AREA_INFO.STORAGE_AREA.getValue());
        }
    };

    @Override
    public void buttonBoxOperation(BtnOpForm btnOpForm){
        log.info("按钮盒请求------------------------》" + JSON.toJSONString(btnOpForm));
        redisCache.setCacheObject(btnOpForm.getIp() + btnOpForm.getOpType(), btnOpForm.getOpType(), 15, TimeUnit.SECONDS);

        String op = buttonBoxAreaMap.get(btnOpForm.getIp());
        String businessType = buttonBoxControlMap.get(btnOpForm.getOpType());
        if (AmsConstant.TASK_BUSINESS_TYPE.TASK_75.getValue().equals(businessType)
                && redisCache.checkIsExist(AmsConstant.AREA_INFO.getKeyByValue(op))) {
            insertAmsTask(op, AmsConstant.TASK_BUSINESS_TYPE.TASK_77.getValue());
            redisCache.setCacheObject(buttonBoxGratingRedisKeyMap.get(op), Boolean.TRUE, 3, TimeUnit.SECONDS);
            redisCache.deleteObject(AmsConstant.AREA_INFO.getKeyByValue(op));
        }
        insertAmsTask(op, businessType);
    }

    private void insertAmsTask(String op, String businessType){
        AmsTask amsTask = new AmsTask();
        amsTask.setTaskNo(IdWorker.getIdStr());
        amsTask.setIkey(Long.parseLong(String.valueOf(CRC16Util.calcCrc16(ByteUtil.string2byteArray(amsTask.getTaskNo())))));
        amsTask.setStFrom(Integer.valueOf(AmsConstant.LOC_MIDDLE_CACHE.toString()));
        amsTask.setStTo(Integer.valueOf(AmsConstant.LOC_SORTATION_CACHE.toString()));
        amsTask.setPriority(1);
        amsTask.setBusinessType(businessType);
        amsTask.setExt1(op);
        amsTaskService.insertAmsTask(amsTask);
    }

}