package com.warewms.common.enums;

public class AutoDoorEnum {

    /**
     * 自动门编号
     */
    public enum AUTO_DOOR_NO {
        /***
         * 靠墙门
         */
        A("1"),
        /***
         * 靠室内
         */
        B("2"),
        /***
         * 靠墙门（第二层）
         */
        C("3"),
        /***
         * 卷帘门
         */
        D("4");

        private String value;

        AUTO_DOOR_NO(String value) {
            this.value = value;
        }

        public String getValue() {
            return value;
        }

        /**
         * 根据值获得枚举类型 switch
         *
         * @param value
         * @return
         */
        public static AUTO_DOOR_NO getByValue(String value) {
            for (AUTO_DOOR_NO code : values()) {
                if (code.getValue().equals(value)) {
                    return code;
                }
            }
            return null;
        }
    }


    /**
     * 自动门命令
     */
    public enum AUTOMATIC_DOOR_CODE {
        /**
         * 开门
         */
        WRITE_OPEN(1, 0, 1, 0),
        /**
         * 关门
         */
        WRITE_CLOSE(1, 1, 1, 0),
        /**
         * 急停
         */
        WRITE_STOP(1, 2, 1, 0),
        /***
         * 开门状态
         */
        RAED_OPEN_STS(1, 10, 2, 0),
        /**
         * 关门状态
         */
        RAED_ClOSE_STS(1, 11, 2, 0);

        /**
         * 从节点id
         */
        private int slaveId;
        /**
         * 地址
         */
        private int offset;
        /**
         * 值
         */
        private int value;
        /**
         * 开关门清除标记
         */
        private int clear;

        AUTOMATIC_DOOR_CODE(int slaveId, int offset, int value, int clear) {
            this.slaveId = slaveId;
            this.offset = offset;
            this.value = value;
            this.clear = clear;
        }

        public int getSlaveId() {
            return slaveId;
        }

        public int getOffset() {
            return offset;
        }

        public int getValue() {
            return value;
        }

        public int getClear() {
            return clear;
        }
    }

}
